<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Calculator;

use Dedi\SyliusEcotaxPlugin\Entity\EcotaxInterface;
use Dedi\SyliusEcotaxPlugin\Specification\AddressHasEcotaxSpecificationInterface;
use Sylius\Component\Core\Exception\MissingChannelConfigurationException;
use Sylius\Component\Core\Model\AddressInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Webmozart\Assert\Assert;

final class EcotaxCalculator implements EcotaxCalculatorInterface
{
    public function __construct(
        private AddressHasEcotaxSpecificationInterface $addressHasEcotaxSpecification,
    ) {
    }

    public function calculate(EcotaxInterface $ecotax, array $context): int
    {
        Assert::keyExists($context, 'channel');

        /** @var ChannelPricingInterface|null $channelPricing */
        $channelPricing = $ecotax->getChannelPricing();
        if (null === $channelPricing) {
            throw new MissingChannelConfigurationException(sprintf(
                'Ecotax %s has no channel pricing defined',
                $ecotax->getId(),
            ));
        }
        $productVariant = $channelPricing->getProductVariant();

        if ($context['channel']->getCode() !== $channelPricing->getChannelCode()) {
            throw new MissingChannelConfigurationException(sprintf(
                'Channel %s has no price defined for product variant %s',
                $context['channel']->getName(),
                $productVariant?->getName(),
            ));
        }

        $address = null;
        if (array_key_exists('address', $context) && $context['address'] instanceof AddressInterface) {
            $address = $context['address'];
        }

        if (null !== $address && !$this->addressHasEcotaxSpecification->isSatisfiedBy($address)) {
            return 0;
        }

        return (int) $ecotax->getValue();
    }
}
