<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Calculator;

use Dedi\SyliusEcotaxPlugin\Model\EcotaxesAwareInterface;
use Dedi\SyliusEcotaxPlugin\Specification\AddressHasEcotaxSpecificationInterface;
use Sylius\Component\Core\Model\AddressInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;
use Webmozart\Assert\Assert;

final class ProductVariantEcotaxCalculator implements ProductVariantEcotaxCalculatorInterface
{
    public function __construct(
        private AddressHasEcotaxSpecificationInterface $addressHasEcotaxSpecification,
        private EcotaxCalculatorInterface $ecotaxCalculator,
    ) {
    }

    public function calculate(ProductVariantInterface $variant, array $context): int
    {
        Assert::keyExists($context, 'channel');

        /** @var ChannelPricingInterface|null $channelPricing */
        $channelPricing = $variant->getChannelPricingForChannel($context['channel']);

        Assert::isInstanceOf($channelPricing, ChannelPricingInterface::class);
        Assert::isInstanceOf($channelPricing, EcotaxesAwareInterface::class);

        $address = null;
        if (array_key_exists('address', $context) && $context['address'] instanceof AddressInterface) {
            $address = $context['address'];
        }

        if (null !== $address && !$this->addressHasEcotaxSpecification->isSatisfiedBy($address)) {
            return 0;
        }

        $ecotaxTotal = 0;
        foreach ($channelPricing->getEcotaxes() as $ecotax) {
            $ecotaxTotal += $this->ecotaxCalculator->calculate($ecotax, $context);
        }

        return $ecotaxTotal;
    }
}
