<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Entity;

use Sylius\Component\Resource\Model\TranslatableTrait;
use Sylius\Component\Resource\Model\TranslationInterface;

class EcotaxType implements EcotaxTypeInterface
{
    use TranslatableTrait {
        __construct as private initializeTranslationsCollection;
        getTranslation as private doGetTranslation;
    }

    private ?int $id = null;

    private ?string $code = null;

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(?string $code): void
    {
        $this->code = $code;
    }

    public function setLabel(?string $label): static
    {
        /** @var EcotaxTypeTranslationInterface $translation */
        $translation = $this->getTranslation();

        $translation->setLabel($label);

        return $this;
    }

    public function getLabel(): ?string
    {
        /** @var EcotaxTypeTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getLabel();
    }

    /**
     * @return EcotaxTypeTranslationInterface
     */
    public function getTranslation(?string $locale = null): TranslationInterface
    {
        /** @var EcotaxTypeTranslationInterface $translation */
        $translation = $this->doGetTranslation($locale);

        return $translation;
    }

    protected function createTranslation(): EcotaxTypeTranslationInterface
    {
        return new EcotaxTypeTranslation();
    }

    public function __toString(): string
    {
        return (string) $this->getLabel();
    }
}
