<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Helper;

use Dedi\SyliusEcotaxPlugin\Calculator\EcotaxCalculatorInterface;
use Dedi\SyliusEcotaxPlugin\Calculator\ProductVariantEcotaxCalculatorInterface;
use Dedi\SyliusEcotaxPlugin\Entity\EcotaxInterface;
use Dedi\SyliusEcotaxPlugin\Model\EcotaxesAwareInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;
use Webmozart\Assert\Assert;

final class EcotaxPriceHelper implements EcotaxPriceHelperInterface
{
    public function __construct(
        private EcotaxCalculatorInterface $calculator,
        private ProductVariantEcotaxCalculatorInterface $variantEcotaxCalculator,
    ) {
    }

    public function getVariantEcotax(ProductVariantInterface $productVariant, array $context): int
    {
        Assert::keyExists($context, 'channel');

        /** @var ChannelPricingInterface|null $channelPricing */
        $channelPricing = $productVariant->getChannelPricingForChannel($context['channel']);
        Assert::isInstanceOf($channelPricing, EcotaxesAwareInterface::class);

        return $this->variantEcotaxCalculator->calculate($productVariant, $context);
    }

    public function getEcotax(EcotaxInterface $ecotax, array $context): int
    {
        Assert::keyExists($context, 'channel');

        return $this->calculator->calculate($ecotax, $context);
    }
}
