<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Model;

use Dedi\SyliusEcotaxPlugin\Entity\EcotaxInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

trait EcotaxesAwareTrait
{
    /**
     * @ORM\OneToMany(
     *    targetEntity="Dedi\SyliusEcotaxPlugin\Entity\Ecotax",
     *    cascade={"persist", "remove"},
     *    mappedBy="channelPricing",
     *    orphanRemoval=true
     * )
     */
    private Collection $ecotaxes;

    public function __construct()
    {
        $this->ecotaxes = new ArrayCollection();
    }

    public function getEcotaxes(): Collection
    {
        return $this->ecotaxes;
    }

    public function hasEcotax(EcotaxInterface $ecotax): bool
    {
        return $this->ecotaxes->contains($ecotax);
    }

    public function getEcotaxByCode(string $code): ?EcotaxInterface
    {
        foreach ($this->ecotaxes as $ecotax) {
            if ($ecotax->getCode() === $code) {
                return $ecotax;
            }
        }

        return null;
    }

    public function addEcotax(EcotaxInterface $ecotax): static
    {
        if (!$this->hasEcotax($ecotax)) {
            $ecotax->setChannelPricing($this);
            $this->ecotaxes->add($ecotax);
        }

        return $this;
    }

    public function removeEcotax(EcotaxInterface $ecotax): static
    {
        if ($this->hasEcotax($ecotax)) {
            $this->ecotaxes->removeElement($ecotax);
            $ecotax->setChannelPricing(null);
        }

        return $this;
    }

    public function removeEcotaxes(): static
    {
        foreach ($this->ecotaxes as $ecotax) {
            $this->ecotaxes->removeElement($ecotax);
            $ecotax->setChannelPricing(null);
        }

        return $this;
    }

    public function getTotalEcotaxes(): int
    {
        if ($this->ecotaxes->isEmpty()) {
            return 0;
        }

        $totalSum = 0;

        /** @var EcotaxInterface $ecotax */
        foreach ($this->ecotaxes as $ecotax) {
            $totalSum += $ecotax->getValue();
        }

        return $totalSum;
    }
}
